@@ -22,17 +22,20 @@
 import logging
 log = logging.getLogger("anaconda")
 
-from gi.repository import GLib, Gtk, Gdk
+from gi.repository import GLib, Gdk, Gtk, TimezoneMap
 
 from pyanaconda.ui.communication import hubQ
 from pyanaconda.ui.common import FirstbootSpokeMixIn
 from pyanaconda.ui.gui import GUIObject
 from pyanaconda.ui.gui.spokes import NormalSpoke
-from pyanaconda.ui.gui.categories.localization import LocalizationCategory
-from pyanaconda.ui.gui.utils import enlightbox, gtk_action_nowait, gtk_call_once
+from pyanaconda.ui.categories.localization import LocalizationCategory
+from pyanaconda.ui.gui.utils import gtk_action_nowait, gtk_action_wait, gtk_call_once, override_cell_property
+from pyanaconda.ui.gui.helpers import GUIDialogInputCheckHandler
+from pyanaconda.ui.helpers import InputCheck
 
-from pyanaconda.i18n import _, N_
-from pyanaconda.timezone import NTP_SERVICE, get_all_regions_and_timezones, is_valid_timezone
+from pyanaconda.i18n import _, CN_
+from pyanaconda.timezone import NTP_SERVICE, get_all_regions_and_timezones, get_timezone, is_valid_timezone
+from pyanaconda.localization import get_xlated_timezone, resolve_date_format
 from pyanaconda import iutil
 from pyanaconda import isys
 from pyanaconda import network
@@ -43,8 +46,9 @@ from pyanaconda import constants
 from pyanaconda.threads import threadMgr, AnacondaThread
 
 import datetime
-import os
+import re
 import threading
+import locale as locale_mod
 
 __all__ = ["DatetimeSpoke"]
 
@@ -52,18 +56,96 @@ SERVER_OK = 0
 SERVER_NOK = 1
 SERVER_QUERY = 2
 
+SERVER_HOSTNAME = 0
+SERVER_POOL = 1
+SERVER_WORKING = 2
+SERVER_USE = 3
+
 DEFAULT_TZ = "America/New_York"
 
-class NTPconfigDialog(GUIObject):
+SPLIT_NUMBER_SUFFIX_RE = re.compile(r'([^0-9]*)([-+])([0-9]+)')
+
+def _compare_regions(reg_xlated1, reg_xlated2):
+    """Compare two pairs of regions and their translations."""
+
+    reg1, xlated1 = reg_xlated1
+    reg2, xlated2 = reg_xlated2
+
+    # sort the Etc timezones to the end
+    if reg1 == "Etc" and reg2 == "Etc":
+        return 0
+    elif reg1 == "Etc":
+        return 1
+    elif reg2 == "Etc":
+        return -1
+    else:
+        # otherwise compare the translated names
+        return locale_mod.strcoll(xlated1, xlated2)
+
+def _compare_cities(city_xlated1, city_xlated2):
+    """Compare two paris of cities and their translations."""
+
+    # if there are "cities" ending with numbers (like GMT+-X), we need to sort
+    # them based on their numbers
+    val1 = city_xlated1[1]
+    val2 = city_xlated2[1]
+
+    match1 = SPLIT_NUMBER_SUFFIX_RE.match(val1)
+    match2 = SPLIT_NUMBER_SUFFIX_RE.match(val2)
+
+    if match1 is None and match2 is None:
+        # no +-X suffix, just compare the strings
+        return locale_mod.strcoll(val1, val2)
+
+    if match1 is None or match2 is None:
+        # one with the +-X suffix, compare the prefixes
+        if match1:
+            prefix, _sign, _suffix = match1.groups()
+            return locale_mod.strcoll(prefix, val2)
+        else:
+            prefix, _sign, _suffix = match2.groups()
+            return locale_mod.strcoll(val1, prefix)
+
+    # both have the +-X suffix
+    prefix1, sign1, suffix1 = match1.groups()
+    prefix2, sign2, suffix2 = match2.groups()
+
+    if prefix1 == prefix2:
+        # same prefixes, let signs determine
+        return cmp(int(sign1 + suffix1), int(sign2 + suffix2))
+    else:
+        # compare prefixes
+        return locale_mod.strcoll(prefix1, prefix2)
+
+def _new_date_field_box(store):
+    """
+    Creates new date field box (a combobox and a label in a horizontal box) for
+    a given store.
+
+    """
+
+    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+    suffix_label = Gtk.Label()
+    renderer = Gtk.CellRendererText()
+    combo = Gtk.ComboBox(model=store)
+    combo.pack_start(renderer, False)
+
+    # idx is column 0, string we want to show is 1
+    combo.add_attribute(renderer, "text", 1)
+
+    box.pack_start(combo, False, False, 0)
+    box.pack_start(suffix_label, False, False, 0)
+
+    return (box, combo, suffix_label)
+
+class NTPconfigDialog(GUIObject, GUIDialogInputCheckHandler):
     builderObjects = ["ntpConfigDialog", "addImage", "serversStore"]
     mainWidgetName = "ntpConfigDialog"
     uiFile = "spokes/datetime_spoke.glade"
 
     def __init__(self, *args):
         GUIObject.__init__(self, *args)
-
-        #used to ensure uniqueness of the threads' names
-        self._threads_counter = 0
+        GUIDialogInputCheckHandler.__init__(self)
 
         #epoch is increased when serversStore is repopulated
         self._epoch = 0
@@ -72,58 +154,94 @@ class NTPconfigDialog(GUIObject):
     @property
     def working_server(self):
         for row in self._serversStore:
-            if row[1] == SERVER_OK and row[2]:
+            if row[SERVER_WORKING] == SERVER_OK and row[SERVER_USE]:
                 #server is checked and working
-                return row[0]
+                return row[SERVER_HOSTNAME]
 
         return None
 
     @property
-    def servers(self):
-        ret = list()
+    def pools_servers(self):
+        pools = list()
+        servers = list()
 
-        for row in self._serversStore:
-            if row[2]:
-                #server checked
-                ret.append(row[0])
+        for used_row in (row for row in self._serversStore if row[SERVER_USE]):
+            if used_row[SERVER_POOL]:
+                pools.append(used_row[SERVER_HOSTNAME])
+            else:
+                servers.append(used_row[SERVER_HOSTNAME])
 
-        return ret
+        return (pools, servers)
 
     def _render_working(self, column, renderer, model, itr, user_data=None):
-        #get the value in the second column
-        value = model[itr][1]
+        value = model[itr][SERVER_WORKING]
 
         if value == SERVER_QUERY:
-            renderer.set_property("stock-id", "gtk-dialog-question")
+            return "dialog-question"
         elif value == SERVER_OK:
-            renderer.set_property("stock-id", "gtk-yes")
+            return "emblem-default"
         else:
-            renderer.set_property("stock-id", "gtk-no")
+            return "dialog-error"
 
     def initialize(self):
         self.window.set_size_request(500, 400)
 
         workingColumn = self.builder.get_object("workingColumn")
         workingRenderer = self.builder.get_object("workingRenderer")
-        workingColumn.set_cell_data_func(workingRenderer, self._render_working)
+        override_cell_property(workingColumn, workingRenderer, "icon-name",
+                self._render_working)
 
         self._serverEntry = self.builder.get_object("serverEntry")
         self._serversStore = self.builder.get_object("serversStore")
 
+        self._addButton = self.builder.get_object("addButton")
+
+        self._poolCheckButton = self.builder.get_object("poolCheckButton")
+
+        # Validate the server entry box
+        self._serverCheck = self.add_check(self._serverEntry, self._validateServer)
+        self._serverCheck.update_check_status()
+
         self._initialize_store_from_config()
 
     def _initialize_store_from_config(self):
         self._serversStore.clear()
 
         if self.data.timezone.ntpservers:
-            for server in self.data.timezone.ntpservers:
-                self._add_server(server)
+            pools, servers = ntp.internal_to_pools_and_servers(self.data.timezone.ntpservers)
         else:
             try:
-                for server in ntp.get_servers_from_config():
-                    self._add_server(server)
+                pools, servers = ntp.get_servers_from_config()
             except ntp.NTPconfigError:
                 log.warning("Failed to load NTP servers configuration")
+                return
+
+        for pool in pools:
+            self._add_server(pool, True)
+        for server in servers:
+            self._add_server(server, False)
+
+
+    def _validateServer(self, inputcheck):
+        server = self.get_input(inputcheck.input_obj)
+
+        # If not set, fail the check to keep the button insensitive, but don't
+        # display an error
+        if not server:
+            return InputCheck.CHECK_SILENT
+
+        (valid, error) = network.sanityCheckHostname(server)
+        if not valid:
+            return "'%s' is not a valid hostname: %s" % (server, error)
+        else:
+            return InputCheck.CHECK_OK
+
+    def set_status(self, inputcheck):
+        # Use GUIDialogInputCheckHandler to set the error message
+        GUIDialogInputCheckHandler.set_status(self, inputcheck)
+
+        # Set the sensitivity of the add button based on the result
+        self._addButton.set_sensitive(inputcheck.check_status == InputCheck.CHECK_OK)
 
     def refresh(self):
         self._serverEntry.grab_focus()
@@ -141,15 +259,10 @@ class NTPconfigDialog(GUIObject):
 
         #OK clicked
         if rc == 1:
-            new_servers = list()
-
-            for row in self._serversStore:
-                #if server checked
-                if row[2]:
-                    new_servers.append(row[0])
+            new_pools, new_servers = self.pools_servers
 
             if flags.can_touch_runtime_system("save NTP servers configuration"):
-                ntp.save_servers_to_config(new_servers)
+                ntp.save_servers_to_config(new_pools, new_servers)
                 iutil.restart_service(NTP_SERVICE)
 
         #Cancel clicked, window destroyed...
@@ -185,8 +298,8 @@ class NTPconfigDialog(GUIObject):
             (store, itr, column, value) = arg_tuple
             store.set_value(itr, column, value)
 
-        orig_hostname = self._serversStore[itr][0]
-        server_working = ntp.ntp_server_working(self._serversStore[itr][0])
+        orig_hostname = self._serversStore[itr][SERVER_HOSTNAME]
+        server_working = ntp.ntp_server_working(self._serversStore[itr][SERVER_HOSTNAME])
 
         #do not let dialog change epoch while we are modifying data
         self._epoch_lock.acquire()
@@ -194,29 +307,27 @@ class NTPconfigDialog(GUIObject):
         #check if we are in the same epoch as the dialog (and the serversStore)
         #and if the server wasn't changed meanwhile
         if epoch_started == self._epoch:
-            actual_hostname = self._serversStore[itr][0]
+            actual_hostname = self._serversStore[itr][SERVER_HOSTNAME]
 
             if orig_hostname == actual_hostname:
                 if server_working:
                     set_store_value((self._serversStore,
-                                    itr, 1, SERVER_OK))
+                                    itr, SERVER_WORKING, SERVER_OK))
                 else:
                     set_store_value((self._serversStore,
-                                    itr, 1, SERVER_NOK))
+                                    itr, SERVER_WORKING, SERVER_NOK))
         self._epoch_lock.release()
 
     @gtk_action_nowait
     def _refresh_server_working(self, itr):
         """ Runs a new thread with _set_server_ok_nok(itr) as a taget. """
 
-        self._serversStore.set_value(itr, 1, SERVER_QUERY)
-        new_thread_name = "AnaNTPserver%d" % self._threads_counter
-        threadMgr.add(AnacondaThread(name=new_thread_name,
+        self._serversStore.set_value(itr, SERVER_WORKING, SERVER_QUERY)
+        threadMgr.add(AnacondaThread(prefix="AnaNTPserver",
                                      target=self._set_server_ok_nok,
                                      args=(itr, self._epoch)))
-        self._threads_counter += 1
 
-    def _add_server(self, server):
+    def _add_server(self, server, pool=False):
         """
         Checks if a given server is a valid hostname and if yes, adds it
         to the list of servers.
@@ -225,34 +336,32 @@ class NTPconfigDialog(GUIObject):
 
         """
 
-        (valid, error) = network.sanityCheckHostname(server)
-        if not valid:
-            log.error("'%s' is not a valid hostname: %s", server, error)
-            return
-
-        for row in self._serversStore:
-            if row[0] == server:
-                #do not add duplicate items
-                return
-
-        itr = self._serversStore.append([server, SERVER_QUERY, True])
+        itr = self._serversStore.append([server, pool, SERVER_QUERY, True])
 
         #do not block UI while starting thread (may take some time)
         self._refresh_server_working(itr)
 
     def on_entry_activated(self, entry, *args):
-        self._add_server(entry.get_text())
-        entry.set_text("")
+        # Check that the input check has passed
+        if self._serverCheck.check_status == InputCheck.CHECK_OK:
+            self._add_server(entry.get_text(), self._poolCheckButton.get_active())
+            entry.set_text("")
+            self._poolCheckButton.set_active(False)
 
     def on_add_clicked(self, *args):
-        self._add_server(self._serverEntry.get_text())
-        self._serverEntry.set_text("")
+        self._serverEntry.emit("activate")
 
     def on_use_server_toggled(self, renderer, path, *args):
         itr = self._serversStore.get_iter(path)
-        old_value = self._serversStore[itr][2]
+        old_value = self._serversStore[itr][SERVER_USE]
 
-        self._serversStore.set_value(itr, 2, not old_value)
+        self._serversStore.set_value(itr, SERVER_USE, not old_value)
+
+    def on_pool_toggled(self, renderer, path, *args):
+        itr = self._serversStore.get_iter(path)
+        old_value = self._serversStore[itr][SERVER_POOL]
+
+        self._serversStore.set_value(itr, SERVER_POOL, not old_value)
 
     def on_server_edited(self, renderer, path, new_text, *args):
         if not path:
@@ -265,11 +374,11 @@ class NTPconfigDialog(GUIObject):
 
         itr = self._serversStore.get_iter(path)
 
-        if self._serversStore[itr][0] == new_text:
+        if self._serversStore[itr][SERVER_HOSTNAME] == new_text:
             return
 
-        self._serversStore.set_value(itr, 0, new_text)
-        self._serversStore.set_value(itr, 1, SERVER_QUERY)
+        self._serversStore.set_value(itr, SERVER_HOSTNAME, new_text)
+        self._serversStore.set_value(itr, SERVER_WORKING, SERVER_QUERY)
 
         self._refresh_server_working(itr)
 
@@ -278,17 +387,23 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
                       "days", "months", "years", "regions", "cities",
                       "upImage", "upImage1", "upImage2", "downImage",
                       "downImage1", "downImage2", "downImage3", "configImage",
-                      "citiesFilter", "daysFilter", "citiesSort", "regionsSort",
+                      "citiesFilter", "daysFilter",
                       "cityCompletion", "regionCompletion",
                       ]
 
     mainWidgetName = "datetimeWindow"
     uiFile = "spokes/datetime_spoke.glade"
+    helpFile = "DateTimeSpoke.xml"
 
     category = LocalizationCategory
 
     icon = "preferences-system-time-symbolic"
-    title = N_("DATE & _TIME")
+    title = CN_("GUI|Spoke", "_TIME & DATE")
+
+    # Hack to get libtimezonemap loaded for GtkBuilder
+    # see https://bugzilla.gnome.org/show_bug.cgi?id=712184
+    _hack = TimezoneMap.TimezoneMap()
+    del(_hack)
 
     def __init__(self, *args):
         NormalSpoke.__init__(self, *args)
@@ -296,9 +411,10 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         # taking values from the kickstart file?
         self._kickstarted = flags.flags.automatedInstall
 
-        self._config_dialog = None
         self._update_datetime_timer_id = None
         self._start_updating_timer_id = None
+        self._shown = False
+        self._tz = None
 
     def initialize(self):
         NormalSpoke.initialize(self)
@@ -308,6 +424,7 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         self._regionsStore = self.builder.get_object("regions")
         self._citiesStore = self.builder.get_object("cities")
         self._tzmap = self.builder.get_object("tzmap")
+        self._dateBox = self.builder.get_object("dateBox")
 
         # we need to know it the new value is the same as previous or not
         self._old_region = None
@@ -315,9 +432,6 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
         self._regionCombo = self.builder.get_object("regionCombobox")
         self._cityCombo = self.builder.get_object("cityCombobox")
-        self._monthCombo = self.builder.get_object("monthCombobox")
-        self._dayCombo = self.builder.get_object("dayCombobox")
-        self._yearCombo = self.builder.get_object("yearCombobox")
 
         self._daysFilter = self.builder.get_object("daysFilter")
         self._daysFilter.set_visible_func(self.existing_date, None)
@@ -325,46 +439,75 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         self._citiesFilter = self.builder.get_object("citiesFilter")
         self._citiesFilter.set_visible_func(self.city_in_region, None)
 
-        self._citiesSort = self.builder.get_object("citiesSort")
-        self._citiesSort.set_sort_column_id(0, Gtk.SortType.ASCENDING)
-
         self._hoursLabel = self.builder.get_object("hoursLabel")
         self._minutesLabel = self.builder.get_object("minutesLabel")
         self._amPmUp = self.builder.get_object("amPmUpButton")
         self._amPmDown = self.builder.get_object("amPmDownButton")
         self._amPmLabel = self.builder.get_object("amPmLabel")
         self._radioButton24h = self.builder.get_object("timeFormatRB")
+        self._amPmRevealer = self.builder.get_object("amPmRevealer")
+
+        # create widgets for displaying/configuring date
+        day_box, self._dayCombo, day_label = _new_date_field_box(self._daysFilter)
+        self._dayCombo.connect("changed", self.on_day_changed)
+        month_box, self._monthCombo, month_label = _new_date_field_box(self._monthsStore)
+        self._monthCombo.connect("changed", self.on_month_changed)
+        year_box, self._yearCombo, year_label = _new_date_field_box(self._yearsStore)
+        self._yearCombo.connect("changed", self.on_year_changed)
+
+        # get the right order for date widgets and respective formats and put
+        # widgets in place
+        widgets, formats = resolve_date_format(year_box, month_box, day_box)
+        for widget in widgets:
+            self._dateBox.pack_start(widget, False, False, 0)
+
+        self._day_format, suffix = formats[widgets.index(day_box)]
+        day_label.set_text(suffix)
+        self._month_format, suffix = formats[widgets.index(month_box)]
+        month_label.set_text(suffix)
+        self._year_format, suffix = formats[widgets.index(year_box)]
+        year_label.set_text(suffix)
 
         self._ntpSwitch = self.builder.get_object("networkTimeSwitch")
 
         self._regions_zones = get_all_regions_and_timezones()
 
-        self._months_nums = dict()
+        # Set the initial sensitivity of the AM/PM toggle based on the time-type selected
+        self._radioButton24h.emit("toggled")
+
+        if not flags.can_touch_runtime_system("modify system time and date"):
+            self._set_date_time_setting_sensitive(False)
+
+        self._config_dialog = NTPconfigDialog(self.data)
+        self._config_dialog.initialize()
 
         threadMgr.add(AnacondaThread(name=constants.THREAD_DATE_TIME,
                                      target=self._initialize))
 
     def _initialize(self):
-        for day in xrange(1, 32):
-            self.add_to_store(self._daysStore, day)
+        # a bit hacky way, but should return the translated strings
+        for i in range(1, 32):
+            day = datetime.date(2000, 1, i).strftime(self._day_format)
+            self.add_to_store_idx(self._daysStore, i, day)
 
-        for i in xrange(1, 13):
-            #a bit hacky way, but should return the translated string
-            #TODO: how to handle language change? Clear and populate again?
-            month = datetime.date(2000, i, 1).strftime('%B')
-            self.add_to_store(self._monthsStore, month)
-            self._months_nums[month] = i
+        for i in range(1, 13):
+            month = datetime.date(2000, i, 1).strftime(self._month_format)
+            self.add_to_store_idx(self._monthsStore, i, month)
 
-        for year in xrange(1990, 2051):
-            self.add_to_store(self._yearsStore, year)
+        for i in range(1990, 2051):
+            year = datetime.date(i, 1, 1).strftime(self._year_format)
+            self.add_to_store_idx(self._yearsStore, i, year)
 
-        for region in self._regions_zones.keys():
-            self.add_to_store(self._regionsStore, region)
+        cities = set()
+        xlated_regions = ((region, get_xlated_timezone(region))
+                          for region in self._regions_zones.keys())
+        for region, xlated in sorted(xlated_regions, cmp=_compare_regions):
+            self.add_to_store_xlated(self._regionsStore, region, xlated)
             for city in self._regions_zones[region]:
-                self.add_to_store(self._citiesStore, city)
+                cities.add((city, get_xlated_timezone(city)))
 
-        if self._radioButton24h.get_active():
-            self._set_amPm_part_sensitive(False)
+        for city, xlated in sorted(cities, cmp=_compare_cities):
+            self.add_to_store_xlated(self._citiesStore, city, xlated)
 
         self._update_datetime_timer_id = None
         if is_valid_timezone(self.data.timezone.timezone):
@@ -375,12 +518,6 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
             self._set_timezone(DEFAULT_TZ)
             self.data.timezone.timezone = DEFAULT_TZ
 
-        if not flags.can_touch_runtime_system("modify system time and date"):
-            self._set_date_time_setting_sensitive(False)
-
-        self._config_dialog = NTPconfigDialog(self.data)
-        self._config_dialog.initialize()
-
         time_init_thread = threadMgr.get(constants.THREAD_TIME_INIT)
         if time_init_thread is not None:
             hubQ.send_message(self.__class__.__name__,
@@ -393,15 +530,19 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
     def status(self):
         if self.data.timezone.timezone:
             if is_valid_timezone(self.data.timezone.timezone):
-                return _("%s timezone") % self.data.timezone.timezone
+                return _("%s timezone") % get_xlated_timezone(self.data.timezone.timezone)
             else:
                 return _("Invalid timezone")
-        elif self._tzmap.get_timezone():
-            return _("%s timezone") % self._tzmap.get_timezone()
         else:
-            return _("Nothing selected")
+            location = self._tzmap.get_location()
+            if location and location.get_property("zone"):
+                return _("%s timezone") % get_xlated_timezone(location.get_property("zone"))
+            else:
+                return _("Nothing selected")
 
     def apply(self):
+        self._shown = False
+
         # we could use self._tzmap.get_timezone() here, but it returns "" if
         # Etc/XXXXXX timezone is selected
         region = self._get_active_region()
@@ -446,13 +587,15 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         return True
 
     def refresh(self):
+        self._shown = True
+
         #update the displayed time
         self._update_datetime_timer_id = GLib.timeout_add_seconds(1,
                                                     self._update_datetime)
         self._start_updating_timer_id = None
 
         if is_valid_timezone(self.data.timezone.timezone):
-            self._set_timezone(self.data.timezone.timezone)
+            self._tzmap.set_timezone(self.data.timezone.timezone)
 
         self._update_datetime()
 
@@ -470,6 +613,7 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
         self._ntpSwitch.set_active(ntp_working)
 
+    @gtk_action_wait
     def _set_timezone(self, timezone):
         """
         Sets timezone to the city/region comboboxes and the timezone map.
@@ -492,14 +636,22 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
         return True
 
+    @gtk_action_nowait
+    def add_to_store_xlated(self, store, item, xlated):
+        store.append([item, xlated])
+
     @gtk_action_nowait
     def add_to_store(self, store, item):
         store.append([item])
 
-    def existing_date(self, model, itr, user_data=None):
-        if not itr:
+    @gtk_action_nowait
+    def add_to_store_idx(self, store, idx, item):
+        store.append([idx, item])
+
+    def existing_date(self, days_model, days_iter, user_data=None):
+        if not days_iter:
             return False
-        day = model[itr][0]
+        day = days_model[days_iter][0]
 
         #days 1-28 are in every month every year
         if day < 29:
@@ -509,16 +661,15 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         months_iter = self._monthCombo.get_active_iter()
         if not months_iter:
             return True
-        month = months_model[months_iter][0]
 
         years_model = self._yearCombo.get_model()
         years_iter = self._yearCombo.get_active_iter()
         if not years_iter:
             return True
-        year = years_model[years_iter][0]
 
         try:
-            datetime.date(year, self._months_nums[month], day)
+            datetime.date(years_model[years_iter][0],
+                          months_model[months_iter][0], day)
             return True
         except ValueError:
             return False
@@ -557,9 +708,9 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
     def _to_amPm(self, hours):
         if hours >= 12:
-            day_phase = "PM"
+            day_phase = _("PM")
         else:
-            day_phase = "AM"
+            day_phase = _("AM")
 
         new_hours = ((hours - 1) % 12) + 1
 
@@ -568,16 +719,16 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
     def _to_24h(self, hours, day_phase):
         correction = 0
 
-        if day_phase == "AM" and hours == 12:
+        if day_phase == _("AM") and hours == 12:
             correction = -12
 
-        elif day_phase == "PM" and hours != 12:
+        elif day_phase == _("PM") and hours != 12:
             correction = 12
 
         return (hours + correction) % 24
 
     def _update_datetime(self):
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(self._tz)
         if self._radioButton24h.get_active():
             self._hoursLabel.set_text("%0.2d" % now.hour)
         else:
@@ -588,8 +739,7 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         self._minutesLabel.set_text("%0.2d" % now.minute)
 
         self._set_combo_selection(self._dayCombo, now.day)
-        self._set_combo_selection(self._monthCombo,
-                            datetime.date(2000, now.month, 1).strftime('%B'))
+        self._set_combo_selection(self._monthCombo, now.month)
         self._set_combo_selection(self._yearCombo, now.year)
 
         #GLib's timer is driven by the return value of the function.
@@ -604,18 +754,18 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
         """
 
+        self._start_updating_timer_id = None
+
         if not flags.can_touch_runtime_system("save system time"):
             return False
 
-        month = self._get_combo_selection(self._monthCombo)
+        month = self._get_combo_selection(self._monthCombo)[0]
         if not month:
             return False
-        month = self._months_nums[month]
 
-        year_str = self._get_combo_selection(self._yearCombo)
-        if not year_str:
+        year = self._get_combo_selection(self._yearCombo)[0]
+        if not year:
             return False
-        year = int(year_str)
 
         hours = int(self._hoursLabel.get_text())
         if not self._radioButton24h.get_active():
@@ -623,14 +773,13 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
 
         minutes = int(self._minutesLabel.get_text())
 
-        day = self._get_combo_selection(self._dayCombo)
+        day = self._get_combo_selection(self._dayCombo)[0]
         #day may be None if there is no such in the selected year and month
         if day:
-            day = int(day)
-            isys.set_system_date_time(year, month, day, hours, minutes)
+            isys.set_system_date_time(year, month, day, hours, minutes, tz=self._tz)
 
         #start the timer only when the spoke is shown
-        if self._update_datetime_timer_id is not None:
+        if self._shown and not self._update_datetime_timer_id:
             self._update_datetime_timer_id = GLib.timeout_add_seconds(1,
                                                         self._update_datetime)
 
@@ -652,13 +801,15 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         """
 
         #do not start timers if the spoke is not shown
-        if self._update_datetime_timer_id is None:
+        if not self._shown:
             self._update_datetime()
             self._save_system_time()
             return
 
         #stop time updating
-        GLib.source_remove(self._update_datetime_timer_id)
+        if self._update_datetime_timer_id:
+            GLib.source_remove(self._update_datetime_timer_id)
+            self._update_datetime_timer_id = None
 
         #stop previous $interval seconds timer (see below)
         if self._start_updating_timer_id:
@@ -695,9 +846,9 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         model = combo.get_model()
         itr = combo.get_active_iter()
         if not itr or not model:
-            return None
+            return None, None
 
-        return model[itr][0]
+        return model[itr][0], model[itr][1]
 
     def _restore_old_city_region(self):
         """Restore stored "old" (or last valid) values."""
@@ -753,21 +904,13 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         minutes_str = "%0.2d" % ((minutes - 1) % 60)
         self._minutesLabel.set_text(minutes_str)
 
-    def on_up_ampm_clicked(self, *args):
+    def on_updown_ampm_clicked(self, *args):
         self._stop_and_maybe_start_time_updating()
 
-        if self._amPmLabel.get_text() == "AM":
-            self._amPmLabel.set_text("PM")
+        if self._amPmLabel.get_text() == _("AM"):
+            self._amPmLabel.set_text(_("PM"))
         else:
-            self._amPmLabel.set_text("AM")
-
-    def on_down_ampm_clicked(self, *args):
-        self._stop_and_maybe_start_time_updating()
-
-        if self._amPmLabel.get_text() == "AM":
-            self._amPmLabel.set_text("PM")
-        else:
-            self._amPmLabel.set_text("AM")
+            self._amPmLabel.set_text(_("AM"))
 
     def on_region_changed(self, combo, *args):
         """
@@ -820,12 +963,23 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
             return
 
         if region == "Etc":
-            # Etc timezones cannot be displayed on the map, so let's set the map
-            # to "" which sets it to "Europe/London" (UTC) without a city pin
-            self._tzmap.set_timezone("", no_signal=True)
+            # Etc timezones cannot be displayed on the map, so let's reset the
+            # location and manually set a highlight with no location pin.
+            self._tzmap.clear_location()
+            if city in ("GMT", "UTC"):
+                offset = 0.0
+            # The tzdb data uses POSIX-style signs for the GMT zones, which is
+            # the opposite of whatever everyone else expects. GMT+4 indicates a
+            # zone four hours west of Greenwich; i.e., four hours before. Reverse
+            # the sign to match the libtimezone map.
+            else:
+                # Take the part after "GMT"
+                offset = -float(city[3:])
+
+            self._tzmap.set_selected_offset(offset)
         else:
             # we don't want the timezone-changed signal to be emitted
-            self._tzmap.set_timezone(timezone, no_signal=True)
+            self._tzmap.set_timezone(timezone)
 
         # update "old" values
         self._old_city = city
@@ -871,10 +1025,14 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         self._stop_and_maybe_start_time_updating(interval=5)
         self._daysFilter.refilter()
 
-    def on_timezone_changed(self, tz_map, timezone):
+    def on_location_changed(self, tz_map, location):
+        if not location:
+            return
+
+        timezone = location.get_property('zone')
         if self._set_timezone(timezone):
             # timezone successfully set
-            os.environ["TZ"] = timezone
+            self._tz = get_timezone(timezone)
             self._update_datetime()
 
     def on_timeformat_changed(self, button24h, *args):
@@ -885,11 +1043,12 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
         if button24h.get_active():
             self._set_amPm_part_sensitive(False)
             new_hours = self._to_24h(hours, amPm)
-
+            self._amPmRevealer.set_reveal_child(False)
         else:
             self._set_amPm_part_sensitive(True)
             new_hours, new_amPm = self._to_amPm(hours)
             self._amPmLabel.set_text(new_amPm)
+            self._amPmRevealer.set_reveal_child(True)
 
         self._hoursLabel.set_text("%0.2d" % new_hours)
 
@@ -901,11 +1060,9 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
     def _show_no_network_warning(self):
         self.set_warning(_("You need to set up networking first if you "\
                            "want to use NTP"))
-        self.window.show_all()
 
     def _show_no_ntp_server_warning(self):
         self.set_warning(_("You have no working NTP server configured"))
-        self.window.show_all()
 
     def on_ntp_switched(self, switch, *args):
         if switch.get_active():
@@ -956,11 +1113,12 @@ class DatetimeSpoke(FirstbootSpokeMixIn, NormalSpoke):
     def on_ntp_config_clicked(self, *args):
         self._config_dialog.refresh()
 
-        with enlightbox(self.window, self._config_dialog.window):
+        with self.main_window.enlightbox(self._config_dialog.window):
             response = self._config_dialog.run()
 
         if response == 1:
-            self.data.timezone.ntpservers = self._config_dialog.servers
+            pools, servers = self._config_dialog.pools_servers
+            self.data.timezone.ntpservers = ntp.pools_servers_to_internal(pools, servers)
 
             if self._config_dialog.working_server is None:
                 self._show_no_ntp_server_warning()
